/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */
extern Config config; /* The system config */

/* End Global Variables */
/*****************************************************************************/ 

/*****************************************************************************/ 
/* Begin function prototypes */

GtkWidget *group_elements(int card, int mixer, Group *group);
GtkWidget *display_volume1(Group *group, int element, void *handle);
GtkWidget *display_switch2(Group *group, int element, void *handle);

/* End function protoypes */
/*****************************************************************************/ 

GtkWidget *create_mixer_page(int card_num, int mixer_num) 
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *frame;
	int i=card_num, j=mixer_num, k=0, l, m;
	int w=1, col;
	
	/* Compute the number of culumns to use */
	//	w = (int)sqrt((double)card[i].mixer[j].info.elements);
	w = card[i].mixer[j].info.elements / card[i].mixer[j].info.groups;

	/* Compute the number of groups in a column */
	col = (card[i].mixer[j].info.groups + w - 1)/ w;

	/* Create the main bounding box */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox);


	/* Make a vertical box for each column, then put that column's worth
	   of mixer groups into the column */
	for(l = 0; l < w; l++) {
		/* Make the vertical box to pack it in */
		vbox = gtk_vbox_new(FALSE, 0);
		gtk_widget_show(vbox);
		gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);

		for(m = 0; m < col && k < card[i].mixer[j].info.groups; m++) {
			/* Make the group frame */
			frame = gtk_frame_new(card[i].mixer[j].groups.pgroups[k].name);
			gtk_widget_show(frame);
			gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
			
			gtk_container_add(GTK_CONTAINER(frame),
					  group_elements(card_num, 
							 mixer_num,
							 &card[i].mixer[j].group[k]));


			/* Now increment the count of which mixer group we're on */
			k++;
		}
	}

	return hbox;
}


GtkWidget *group_elements(int card_num, int mixer, Group *group)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *box;
	char thor[128];
	int i, j;
	snd_mixer_element_t test;

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);

	test = group->element[0];
	test.data.volume1 = test.data.volume1;

	for(i = 0; i < group->group.elements; i++) {
		/* Each element gets its own horizontal box */
		hbox=gtk_hbox_new(FALSE, 0);
		gtk_widget_show(hbox);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		
		snprintf(thor, 128, "%s routed to the %s", 
			 group->group.pelements[i].name, 
			 group->routes[i].proutes[0].name);

		label = gtk_label_new(thor);
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);


		switch(group->group.pelements[i].type){

		case SND_MIXER_ETYPE_VOLUME1:
			gtk_box_pack_end(GTK_BOX(hbox), 
					 display_volume1(group, i, 
							 card[card_num].mixer[mixer].handle), 
					 FALSE, FALSE, 0);
			
			break;
		case SND_MIXER_ETYPE_SWITCH2:
			gtk_box_pack_end(GTK_BOX(hbox),
					 display_switch2(group, i, 
							 card[card_num].mixer[mixer].handle),
					 FALSE, FALSE, 0);
			break;
		}
	}



#if 0
	snprintf(test, 64, "%i", group->group.elements);
	label=gtk_label_new(test);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

#endif
	return vbox;
}

GtkWidget *display_switch2(Group *group, int element, void *handle)
{
	GtkWidget *button;
	int i,j;

	i = element;

	if(!group) {
		printf("Group isn't initialized!\n");
		return NULL;
	}

	button = gtk_check_button_new();

	if(group->element)
		if(group->element[i].data.switch2.sw) {
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
		}

	gtk_widget_show(button);

	if(group->gtk) {
		group->gtk[i].interface = calloc(1, sizeof(GtkWidget *));
		group->gtk[i].interface[0] = button;
	} else {
		printf("Something wasn't initialized properly.\n");
	}

	return button;
}

GtkWidget *display_volume1(Group *group, int element, void *handle)
{
	GtkWidget *box;
	int i,j;

	i = element;

	box = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(box);

	group->gtk[i].adjust = calloc(group->element[i].data.volume1.voices, 
				      sizeof(GtkObject *));
	group->gtk[i].interface = calloc(group->element[i].data.volume1.voices, 
					 sizeof(GtkWidget *));
	
	for(j=0; j < group->element[i].data.volume1.voices; j++) {
		group->gtk[i].adjust[j] = 
			gtk_adjustment_new(group->element[i].data.volume1.pvoices[j],
					   group->einfo[i].data.volume1.prange[0].min,
					   group->einfo[i].data.volume1.prange[0].max,
					   1.0,
					   3.0,
					   0.0);
		
		group->gtk[i].interface[j] = 
			gtk_hscale_new(GTK_ADJUSTMENT(group->gtk[i].adjust[j]));

		gtk_signal_connect(GTK_OBJECT(group->gtk[i].adjust[j]),
				   "value_changed", 
				   GTK_SIGNAL_FUNC (adjust_volume1),
				   create_cb_data(group, handle, element, j));

/* 		gtk_scale_set_draw_value(GTK_SCALE(group->gtk[i].interface[j]), */
/* 					 FALSE); */

		gtk_scale_set_value_pos(GTK_SCALE(group->gtk[i].interface[j]), 
					GTK_POS_RIGHT);

/* 		gtk_widget_set_usize(group->gtk[i].interface[j], 150, -1); */

		gtk_widget_show(group->gtk[i].interface[j]);
		gtk_box_pack_start(GTK_BOX(box), 
				   group->gtk[i].interface[j], 
				   FALSE, FALSE, 0);
	}
	


	return box;
}
